var fs = require("fs"),
    Buffer = require('buffer').Buffer,
    Deferred = require('Deferred'),
    Long = require('long');


function Hash(){}


/**
 *
 * Get the file descriptor
 * 
 * @param  {String} filename
 * @return {Promise(fd, size)}
 */
var getFileDescriptor = function(filename){

    var dfd = new Deferred();

    fs.open(filename, 'r', function(status, fd) {

        fs.stat(filename, function(err, stats){
            dfd.resolve(fd, stats.size);    
        });

    });

    return dfd.promise();

};


/**
 * Sums every long that is in the chunk
 * 
 * @param  {fd} fd
 * @param  {Number} start
 * @param  {Number} length
 * @return {Promise(Long)}
 * 
 */
var longSumChunk = function(fd, start, length){

    var dfd = new Deferred();

    /// Read the buffer
    var buffer = new Buffer(length);
    fs.read(fd, buffer, 0, length, start, function(err, num) {

        if(err){ dfd.resolve(new Long()); }

        /// get the number of bytes readed
        buffer = buffer.slice(0, num);

        /// sum the 
        var hNumber = new Long();
        for(var i=0; i<buffer.length; i+=8){

           var low = buffer.readUInt32LE(i, true);
           var high = buffer.readUInt32LE(i+4, true);

           var n = new Long(low, high);
           hNumber = hNumber.add(n);
        }


        dfd.resolve(hNumber);

    });

    return dfd.promise();

};

/*
 * Get Hash from file
 *
 */
Hash.prototype.getHash = function(filename){
    
    var dfd = new Deferred(),
        HASH_CHUNK_SIZE = 64 * 1024;


    /// Pyrammid of doom!
    getFileDescriptor(filename)
    .then(
        function(fd, size){

            if(size<HASH_CHUNK_SIZE)
                HASH_CHUNK_SIZE = file.size;

            /// get the head
            longSumChunk(fd, 0, HASH_CHUNK_SIZE)
            .then(
                function(head){

                    var start = size-HASH_CHUNK_SIZE;
                    if(start < 0)
                       start = 0;

                    /// get the tail
                    longSumChunk(fd, start, HASH_CHUNK_SIZE)
                    .then(
                        function(tail){

                            // sum all values            
                            var sum = head
                                    .add(tail)
                                    .add(new Long(size));

                            // convert value to unsigned
                            var sumHex = sum.toUnsigned().toString(16);
                            
                            dfd.resolve(sumHex);

                        }
                    );

                }
            );


        }
    );
    
    return dfd.promise();

};


module.exports = new Hash();